package cn.com.infosec.crypto.tls;

import cn.com.infosec.crypto.BlockCipher;
import cn.com.infosec.crypto.Digest;
import cn.com.infosec.crypto.params.KeyParameter;
import cn.com.infosec.crypto.params.ParametersWithIV;

public class TlsBlockCipherCipherSuite extends TlsCipherSuite {
   private BlockCipher encryptCipher;
   private BlockCipher decryptCipher;
   private Digest writeDigest;
   private Digest readDigest;
   private int cipherKeySize;
   private short keyExchange;
   private TlsMac writeMac;
   private TlsMac readMac;

   protected TlsBlockCipherCipherSuite(BlockCipher encrypt, BlockCipher decrypt, Digest writeDigest, Digest readDigest, int cipherKeySize, short keyExchange) {
      this.encryptCipher = encrypt;
      this.decryptCipher = decrypt;
      this.writeDigest = writeDigest;
      this.readDigest = readDigest;
      this.cipherKeySize = cipherKeySize;
      this.keyExchange = keyExchange;
   }

   private void initCipher(boolean forEncryption, BlockCipher cipher, byte[] key_block, int key_size, int key_offset, int iv_offset) {
      BlockCipher var10000 = cipher;
      KeyParameter var7;
      var7 = new KeyParameter.<init>(key_block, key_offset, key_size);
      ParametersWithIV key_size1;
      ParametersWithIV var10001 = key_size1 = new ParametersWithIV;
      int cipher1 = cipher.getBlockSize();
      var10001.<init>(var7, key_block, iv_offset, cipher1);
      var10000.init(forEncryption, key_size1);
   }

   protected void init(byte[] ms, byte[] cr, byte[] sr) {
      TlsBlockCipherCipherSuite var10000 = this;
      byte[] var10003 = ms;
      byte[] var10004 = sr;
      byte[] var10005 = sr;
      byte[] var10006 = cr;
      byte[] var10007 = cr;
      ms = new byte[this.cipherKeySize * 2 + this.writeDigest.getDigestSize() * 2 + this.encryptCipher.getBlockSize() * 2];
      cr = new byte[cr.length + sr.length];
      int sr1 = sr.length;
      int var4 = var10007.length;
      System.arraycopy(var10006, 0, cr, sr1, var4);
      sr1 = var10005.length;
      System.arraycopy(var10004, 0, cr, 0, sr1);
      TlsUtils.PRF(var10003, TlsUtils.toByteArray("key expansion"), cr, ms);
      Digest var6 = this.writeDigest;
      int cr1 = var6.getDigestSize();
      this.writeMac = new TlsMac(var6, ms, 0, cr1);
      int var10001 = cr1 = this.writeDigest.getDigestSize() + 0;
      Digest var8 = this.readDigest;
      sr1 = var8.getDigestSize();
      this.readMac = new TlsMac(var8, ms, cr1, sr1);
      var10001 = cr1 = var10001 + this.readDigest.getDigestSize();
      BlockCipher sr2 = this.encryptCipher;
      int var5 = cr1 + (var4 = this.cipherKeySize) * 2;
      this.initCipher(true, sr2, ms, var4, cr1, var5);
      var10001 = sr1 = var10001 + (cr1 = this.cipherKeySize);
      BlockCipher var7 = this.decryptCipher;
      var4 = var10001 + cr1 + var7.getBlockSize();
      var10000.initCipher(false, var7, ms, cr1, sr1, var4);
   }

   protected byte[] encodePlaintext(short type, byte[] plaintext, int offset, int len) {
      int var5;
      int var6 = (var5 = this.encryptCipher.getBlockSize()) - (len + this.writeMac.getSize() + 1) % var5;
      int var7;
      byte[] var8;
      System.arraycopy(plaintext, offset, var8 = new byte[var7 = len + this.writeMac.getSize() + var6 + 1], 0, len);
      byte[] var10001 = this.writeMac.calculateMac((short)type, plaintext, offset, len);
      type = var10001.length;
      System.arraycopy(var10001, 0, var8, len, type);
      type = len + var10001.length;

      for(int plaintext1 = 0; plaintext1 <= var6; ++plaintext1) {
         offset = plaintext1 + type;
         var8[offset] = (byte)var6;
      }

      for(type = 0; type < var7; type += var5) {
         this.encryptCipher.processBlock(var8, type, var8, type);
      }

      return var8;
   }

   protected byte[] decodeCiphertext(short type, byte[] ciphertext, int offset, int len, TlsProtocolHandler handler) {
      int var6 = this.decryptCipher.getBlockSize();
      boolean var7 = false;

      int var8;
      for(var8 = 0; var8 < len; var8 += var6) {
         this.decryptCipher.processBlock(ciphertext, var8 + offset, ciphertext, var8 + offset);
      }

      int var9;
      byte var10;
      if ((var9 = offset + len - 1) - (var10 = ciphertext[var9]) < 0) {
         boolean var14 = true;
         var10 = 0;
         var7 = var14;
      } else {
         for(var6 = 0; var6 <= var10; ++var6) {
            if (ciphertext[var9 - var6] != var10) {
               var7 = true;
            }
         }
      }

      var8 = len - this.readMac.getSize() - var10 - 1;
      byte[] var11 = this.readMac.calculateMac(type, ciphertext, offset, var8);

      for(int var12 = 0; var12 < var11.length; ++var12) {
         if (ciphertext[offset + var8 + var12] != var11[var12]) {
            var7 = true;
         }
      }

      if (var7) {
         handler.failWithError((short)2, (short)20);
      }

      byte[] var13;
      byte[] var10000 = var13 = new byte[var8];
      System.arraycopy(ciphertext, offset, var13, 0, var8);
      return var10000;
   }

   protected short getKeyExchangeAlgorithm() {
      return this.keyExchange;
   }
}
